/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package blending;

import java.awt.Composite;
import java.awt.CompositeContext;
import java.awt.RenderingHints;
import java.awt.image.ColorModel;

/**
 * The BlendingComposite, along with abstract class BlendingContext,
 * defines the methods to compose a draw primitive with the underlying graphics area.
 * <p>
 * Any implementation of BlendingContext is available to be used by this Blending composite
 * </p>
 * 
 * @see BlendingComposite#GRAIN_MERGE
 * @see BlendingComposite#MULTIPLY
 * @see BlendingComposite#COLOR_OVERLAY
 */
public class BlendingComposite implements Composite {

	/**
	 * Grain Merge mode merges a grain layer into the current layer,
	 * leaving a grainy version of the original layer. It adds the
	 * pixel values of the upper and lower layers together and subtracts 128.
	 * <p>
	 * The equation is: R = B + T - 128
	 * </p>
	 */
	public static final int GRAIN_MERGE = 0;

	/**
	 * Multiply mode multiplies the pixel values of the upper layer
	 * with those of the layer below it and then divides the result
	 * by 255. The result is usually a darker image.
	 * <p>
	 * The equation is: R = T × B ÷ 255
	 * </p>
	 */
	public static final int MULTIPLY = 1;

	/**
	 * ColorOverlay mode uses only the pixel color values of the
	 * upper layer with the pixel alpha value of the layer below.
	 */
	public static final int COLOR_OVERLAY = 2;

	/**
	 * The resulting BlendingContext to be used by this instance
	 */
	private CompositeContext blendingContext;

	/**
	 * The resulting Blending mode to be used by this instance
	 */
	private final int blendingMode;	

	/**
	 * Create a new instance of BlendingComposite
	 * using the given BlentingMode value
	 * <p>
	 * The value is one of the constants defined in
	 * class <code>BlendingComposite</code> which is applicable
	 * to instances of this class. The class description lists
	 * the constants that are applicable to the class.
	 * </p>
	 * 
	 * @param blendingMode the BlendingMode value to be used
	 */
	public BlendingComposite(int blendingMode) {
		switch (blendingMode) {
		case GRAIN_MERGE:
			blendingContext = new GrainMergeBlendingContext();
			this.blendingMode = GRAIN_MERGE;
			break;
		case MULTIPLY:
			blendingContext = new MultiplyBlendingContext();
			this.blendingMode = MULTIPLY;
			break;
		case COLOR_OVERLAY:
			blendingContext = new ColorOverlayBlendingContext();
			this.blendingMode = COLOR_OVERLAY;
			break;

		default:
			blendingContext = null;
		this.blendingMode = -1;
		break;
		}
	}

	public CompositeContext createContext(ColorModel srcColorModel, ColorModel dstColorModel, RenderingHints hints) {
		return blendingContext;
	}

	/**
	 * Converts a string representation of a given blending mode
	 * into the blending mode INT constant value
	 * @param blendingMode the string for the blending mode
	 * @return it's constant value or -1 if match is not found
	 */
	public static int parse(String blendingMode) {
		if (blendingMode.equals("GRAIN_MERGE")) {
			return 0;
		} else if (blendingMode.equals("MULTIPLY")) {
			return 1;
		} else if (blendingMode.equals("COLOR_OVERLAY")) {
			return 2;
		}
		return -1;
	}

	/**
	 * Return the type of this instance blending mode,
	 * which could be one of the GRAIN_MERGE, MULTIPLY
	 * or COLOR_OVERLAY values
	 * @return the blending mode in use by this instance
	 */
	public int getBlendingMode() {
		return blendingMode;
	}

}